#include "EditManager.h"
#include "Window.h"


bool EditManager::lineOk(const Line& l) const {
  Line l2;
  
  for(size_t i = 0; i+2 < newRoom.size(); i++) {
    l2.setVertex1(newRoom.at(i));
    l2.setVertex2(newRoom.at(i+1));
    
    if(l.intersects(l2, NULL) == INTERSECTION_SEGMENT_SEGMENT) return false;
  }
  
  if(newRoom.size() > 1) {
    l2.setVertex1(newRoom.at(newRoom.size()-2));
    l2.setVertex2(newRoom.back());
    if(l2.contains(l.getVertex2())) return false;
  }
  
  for(Level::iterator room = window->getLevel().begin(); room != window->getLevel().end(); room++) {
    if((*room)->isOfType("Room") && ((Room*)&**room)->intersects(l))
      return false;
  }
  
  return true;
}


EditManager::EditManager(Window *window) {
  this->window = window;
  
  selectedObject = NULL;
  mode = VIEW;
  
  hoveredObject = NULL;
  
  hasHoveredVertex = false;
}

void EditManager::redraw() {
  window->redraw();
}

void EditManager::addRoom() {
  if(mode == VIEW) {
    newRoom = Room();
    mode = ADD;
    
    selectedObject = &newRoom;
    
    window->update();
  }
}

void EditManager::finishRoom() {
  if(mode == ADD) {
    mode = VIEW;
    
    if(newRoom.size() > 2 && polygonOk(newRoom)) {
      newRoom.setName(idManager.generate("room"));
      window->getLevel().push_back(SharedPtr<LevelObject>(new Room(newRoom)));
      selectedObject = &*window->getLevel().back();
    }
    else {
      selectedObject = NULL;
    }
    
    window->update();
  }
}

void EditManager::addVertex(const Vertex &v) {
  if(mode != ADD)
    return;
  
  newRoom.push_back(v);
  window->update();
}

Vertex* EditManager::getHoveredVertex() {
  if(hasHoveredVertex) return &hoveredVertex;
  else return NULL;
}

void EditManager::setHoveredVertex(Vertex *v) {
  if(v) {
    hasHoveredVertex = true;
    hoveredVertex = *v;
    
    hoveredObject = NULL;
    
    for(Level::iterator object = window->getLevel().begin(); object != window->getLevel().end(); object++) {
      if((*object)->hit(*v)) {
        if(hoveredObject && (*object)->getPriority() < hoveredObject->getPriority())
          continue;
        
        hoveredObject = &**object;
      }
    }
  }
  else {
    hasHoveredVertex = false;
    hoveredObject = NULL;
  }
  
  window->update();
}

void EditManager::buttonPress(unsigned int button) {
  switch(button) {
    case 1:
      if(!hasHoveredVertex)
        break;
      
      switch(mode) {
        case VIEW:
          selectedObject = hoveredObject;
          break;
          
        case ADD:
          if(vertexOk(hoveredVertex))
            addVertex(hoveredVertex);
      }
      
      window->update();
  }
}

bool EditManager::vertexOk(const Vertex& v) const {
  Line l;
  
  for(Level::iterator room = window->getLevel().begin(); room != window->getLevel().end(); room++) {
    if((*room)->isOfType("Room") && ((Room*)&**room)->contains(v)) return false;
  }
  
  if(newRoom.empty())
    return true;
  
  l.setVertex1(newRoom.back());
  l.setVertex2(v);
  
  return lineOk(l);
}

bool EditManager::polygonOk(const Polygon& polygon) const {
  Line l, l2;
  
  if(polygon.empty()) return false;
  
  for(Level::iterator object = window->getLevel().begin(); object != window->getLevel().end(); object++) {
    if(!(*object)->isOfType("Room"))
      continue;
    
    Room *room = (Room*)&**object;
    
    if(room->empty()) continue;
    
    if(room->contains(polygon.front()))
      return false;
    
    if(polygon.contains(room->front()))
      return false;
  }
  
  if(polygon.size() == 1)
    return true;
  
  if(!polygon.isSimple())
    return false;
  
  for(Polygon::const_iterator it = polygon.begin(); it != polygon.end(); it++) {
    Polygon::const_iterator it2 = it+1;
    if(it2 == polygon.end()) it2 = polygon.begin();
    
    l.setVertex1(*it);
    l.setVertex2(*it2);
    
    for(Level::iterator room = window->getLevel().begin(); room != window->getLevel().end(); room++) {
      if((*room)->isOfType("Room") && ((Room*)&**room)->intersects(l))
        return false;
    }
  }
  
  return true;
}
